@@ -3,7 +3,7 @@
 import subprocess
 from pathlib import Path
 
-from ultralytics.yolo.utils import ROOT, SETTINGS
+from ultralytics.yolo.utils import LINUX, ROOT, SETTINGS, checks
 
 MODEL = Path(SETTINGS['weights_dir']) / 'yolov8n'
 CFG = 'yolov8n'
@@ -22,7 +22,7 @@ def test_special_modes():
 
 # Train checks ---------------------------------------------------------------------------------------------------------
 def test_train_det():
-    run(f'yolo train detect model={CFG}.yaml data=coco8.yaml imgsz=32 epochs=1')
+    run(f'yolo train detect model={CFG}.yaml data=coco8.yaml imgsz=32 epochs=1 v5loader')
 
 
 def test_train_seg():
@@ -30,7 +30,7 @@ def test_train_seg():
 
 
 def test_train_cls():
-    run(f'yolo train classify model={CFG}-cls.yaml data=mnist160 imgsz=32 epochs=1')
+    run(f'yolo train classify model={CFG}-cls.yaml data=imagenet10 imgsz=32 epochs=1')
 
 
 # Val checks -----------------------------------------------------------------------------------------------------------
@@ -43,23 +43,24 @@ def test_val_segment():
 
 
 def test_val_classify():
-    run(f'yolo val classify model={MODEL}-cls.pt data=mnist160 imgsz=32')
+    run(f'yolo val classify model={MODEL}-cls.pt data=imagenet10 imgsz=32')
 
 
 # Predict checks -------------------------------------------------------------------------------------------------------
 def test_predict_detect():
-    run(f"yolo predict detect model={MODEL}.pt source={ROOT / 'assets'} imgsz=32")
-    run(f"yolo predict detect model={MODEL}.pt source=https://ultralytics.com/images/bus.jpg imgsz=32")
-    run(f"yolo predict detect model={MODEL}.pt source=https://ultralytics.com/assets/decelera_landscape.mov imgsz=32")
-    run(f"yolo predict detect model={MODEL}.pt source=https://ultralytics.com/assets/decelera_portrait.mov imgsz=32")
+    run(f"yolo predict model={MODEL}.pt source={ROOT / 'assets'} imgsz=32 save save_crop save_txt")
+    if checks.check_online():
+        run(f'yolo predict model={MODEL}.pt source=https://ultralytics.com/images/bus.jpg imgsz=32')
+        run(f'yolo predict model={MODEL}.pt source=https://ultralytics.com/assets/decelera_landscape_min.mov imgsz=32')
+        run(f'yolo predict model={MODEL}.pt source=https://ultralytics.com/assets/decelera_portrait_min.mov imgsz=32')
 
 
 def test_predict_segment():
-    run(f"yolo predict segment model={MODEL}-seg.pt source={ROOT / 'assets'} imgsz=32")
+    run(f"yolo predict model={MODEL}-seg.pt source={ROOT / 'assets'} imgsz=32 save")
 
 
 def test_predict_classify():
-    run(f"yolo predict classify model={MODEL}-cls.pt source={ROOT / 'assets'} imgsz=32")
+    run(f"yolo predict model={MODEL}-cls.pt source={ROOT / 'assets'} imgsz=32 save")
 
 
 # Export checks --------------------------------------------------------------------------------------------------------
@@ -73,3 +74,8 @@ def test_export_segment_torchscript():
 
 def test_export_classify_torchscript():
     run(f'yolo export model={MODEL}-cls.pt format=torchscript')
+
+
+def test_export_detect_edgetpu(enabled=False):
+    if enabled and LINUX:
+        run(f'yolo export model={MODEL}.pt format=edgetpu')
