@@ -18,7 +18,6 @@ from typing import Union
 import cv2
 import numpy as np
 import pandas as pd
-import requests
 import torch
 import yaml
 
@@ -27,13 +26,14 @@ from ultralytics import __version__
 # Constants
 FILE = Path(__file__).resolve()
 ROOT = FILE.parents[2]  # YOLO
-DEFAULT_CFG_PATH = ROOT / "yolo/cfg/default.yaml"
+DEFAULT_CFG_PATH = ROOT / 'yolo/cfg/default.yaml'
 RANK = int(os.getenv('RANK', -1))
 NUM_THREADS = min(8, max(1, os.cpu_count() - 1))  # number of YOLOv5 multiprocessing threads
 AUTOINSTALL = str(os.getenv('YOLO_AUTOINSTALL', True)).lower() == 'true'  # global auto-install mode
 VERBOSE = str(os.getenv('YOLO_VERBOSE', True)).lower() == 'true'  # global verbose mode
 TQDM_BAR_FORMAT = '{l_bar}{bar:10}{r_bar}'  # tqdm bar format
 LOGGING_NAME = 'ultralytics'
+MACOS, LINUX, WINDOWS = (platform.system() == x for x in ['Darwin', 'Linux', 'Windows'])  # environment booleans
 HELP_MSG = \
     """
     Usage examples for running YOLOv8:
@@ -96,6 +96,7 @@ HELP_MSG = \
 torch.set_printoptions(linewidth=320, precision=5, profile='long')
 np.set_printoptions(linewidth=320, formatter={'float_kind': '{:11.5g}'.format})  # format short g, %precision=5
 pd.options.display.max_columns = 10
+pd.options.display.width = 120
 cv2.setNumThreads(0)  # prevent OpenCV from multithreading (incompatible with PyTorch DataLoader)
 os.environ['NUMEXPR_MAX_THREADS'] = str(NUM_THREADS)  # NumExpr max threads
 os.environ['CUBLAS_WORKSPACE_CONFIG'] = ':4096:8'  # for deterministic training
@@ -110,7 +111,7 @@ class IterableSimpleNamespace(SimpleNamespace):
         return iter(vars(self).items())
 
     def __str__(self):
-        return '\n'.join(f"{k}={v}" for k, v in vars(self).items())
+        return '\n'.join(f'{k}={v}' for k, v in vars(self).items())
 
     def __getattr__(self, attr):
         name = self.__class__.__name__
@@ -221,11 +222,10 @@ def is_jupyter():
     Returns:
         bool: True if running inside a Jupyter Notebook, False otherwise.
     """
-    try:
+    with contextlib.suppress(Exception):
         from IPython import get_ipython
         return get_ipython() is not None
-    except ImportError:
-        return False
+    return False
 
 
 def is_docker() -> bool:
@@ -287,11 +287,7 @@ def is_pytest_running():
     Returns:
         (bool): True if pytest is running, False otherwise.
     """
-    try:
-        import sys
-        return "pytest" in sys.modules
-    except ImportError:
-        return False
+    return ('PYTEST_CURRENT_TEST' in os.environ) or ('pytest' in sys.modules) or ('pytest' in Path(sys.argv[0]).stem)
 
 
 def is_github_actions_ci() -> bool:
@@ -338,7 +334,7 @@ def get_git_origin_url():
     """
     if is_git_dir():
         with contextlib.suppress(subprocess.CalledProcessError):
-            origin = subprocess.check_output(["git", "config", "--get", "remote.origin.url"])
+            origin = subprocess.check_output(['git', 'config', '--get', 'remote.origin.url'])
             return origin.decode().strip()
     return None  # if not git dir or on error
 
@@ -352,27 +348,11 @@ def get_git_branch():
     """
     if is_git_dir():
         with contextlib.suppress(subprocess.CalledProcessError):
-            origin = subprocess.check_output(["git", "rev-parse", "--abbrev-ref", "HEAD"])
+            origin = subprocess.check_output(['git', 'rev-parse', '--abbrev-ref', 'HEAD'])
             return origin.decode().strip()
     return None  # if not git dir or on error
 
 
-def get_latest_pypi_version(package_name='ultralytics'):
-    """
-    Returns the latest version of a PyPI package without downloading or installing it.
-
-    Parameters:
-        package_name (str): The name of the package to find the latest version for.
-
-    Returns:
-        str: The latest version of the package.
-    """
-    response = requests.get(f"https://pypi.org/pypi/{package_name}/json")
-    if response.status_code == 200:
-        return response.json()["info"]["version"]
-    return None
-
-
 def get_default_args(func):
     """Returns a dictionary of default arguments for a function.
 
@@ -396,18 +376,15 @@ def get_user_config_dir(sub_dir='Ultralytics'):
     Returns:
         Path: The path to the user config directory.
     """
-    # Get the operating system name
-    os_name = platform.system()
-
     # Return the appropriate config directory for each operating system
-    if os_name == 'Windows':
+    if WINDOWS:
         path = Path.home() / 'AppData' / 'Roaming' / sub_dir
-    elif os_name == 'Darwin':  # macOS
+    elif MACOS:  # macOS
         path = Path.home() / 'Library' / 'Application Support' / sub_dir
-    elif os_name == 'Linux':
+    elif LINUX:
         path = Path.home() / '.config' / sub_dir
     else:
-        raise ValueError(f'Unsupported operating system: {os_name}')
+        raise ValueError(f'Unsupported operating system: {platform.system()}')
 
     # GCP and AWS lambda fix, only /tmp is writeable
     if not is_dir_writeable(str(path.parent)):
@@ -424,33 +401,33 @@ USER_CONFIG_DIR = get_user_config_dir()  # Ultralytics settings dir
 
 def emojis(string=''):
     # Return platform-dependent emoji-safe version of string
-    return string.encode().decode('ascii', 'ignore') if platform.system() == 'Windows' else string
+    return string.encode().decode('ascii', 'ignore') if WINDOWS else string
 
 
 def colorstr(*input):
     # Colors a string https://en.wikipedia.org/wiki/ANSI_escape_code, i.e.  colorstr('blue', 'hello world')
-    *args, string = input if len(input) > 1 else ("blue", "bold", input[0])  # color arguments, string
+    *args, string = input if len(input) > 1 else ('blue', 'bold', input[0])  # color arguments, string
     colors = {
-        "black": "\033[30m",  # basic colors
-        "red": "\033[31m",
-        "green": "\033[32m",
-        "yellow": "\033[33m",
-        "blue": "\033[34m",
-        "magenta": "\033[35m",
-        "cyan": "\033[36m",
-        "white": "\033[37m",
-        "bright_black": "\033[90m",  # bright colors
-        "bright_red": "\033[91m",
-        "bright_green": "\033[92m",
-        "bright_yellow": "\033[93m",
-        "bright_blue": "\033[94m",
-        "bright_magenta": "\033[95m",
-        "bright_cyan": "\033[96m",
-        "bright_white": "\033[97m",
-        "end": "\033[0m",  # misc
-        "bold": "\033[1m",
-        "underline": "\033[4m"}
-    return "".join(colors[x] for x in args) + f"{string}" + colors["end"]
+        'black': '\033[30m',  # basic colors
+        'red': '\033[31m',
+        'green': '\033[32m',
+        'yellow': '\033[33m',
+        'blue': '\033[34m',
+        'magenta': '\033[35m',
+        'cyan': '\033[36m',
+        'white': '\033[37m',
+        'bright_black': '\033[90m',  # bright colors
+        'bright_red': '\033[91m',
+        'bright_green': '\033[92m',
+        'bright_yellow': '\033[93m',
+        'bright_blue': '\033[94m',
+        'bright_magenta': '\033[95m',
+        'bright_cyan': '\033[96m',
+        'bright_white': '\033[97m',
+        'end': '\033[0m',  # misc
+        'bold': '\033[1m',
+        'underline': '\033[4m'}
+    return ''.join(colors[x] for x in args) + f'{string}' + colors['end']
 
 
 def remove_ansi_codes(string):
@@ -471,21 +448,21 @@ def set_logging(name=LOGGING_NAME, verbose=True):
     rank = int(os.getenv('RANK', -1))  # rank in world for Multi-GPU trainings
     level = logging.INFO if verbose and rank in {-1, 0} else logging.ERROR
     logging.config.dictConfig({
-        "version": 1,
-        "disable_existing_loggers": False,
-        "formatters": {
+        'version': 1,
+        'disable_existing_loggers': False,
+        'formatters': {
             name: {
-                "format": "%(message)s"}},
-        "handlers": {
+                'format': '%(message)s'}},
+        'handlers': {
             name: {
-                "class": "logging.StreamHandler",
-                "formatter": name,
-                "level": level}},
-        "loggers": {
+                'class': 'logging.StreamHandler',
+                'formatter': name,
+                'level': level}},
+        'loggers': {
             name: {
-                "level": level,
-                "handlers": [name],
-                "propagate": False}}})
+                'level': level,
+                'handlers': [name],
+                'propagate': False}}})
 
 
 class TryExcept(contextlib.ContextDecorator):
@@ -526,36 +503,32 @@ def set_sentry():
                 return None  # do not send event
 
         event['tags'] = {
-            "sys_argv": sys.argv[0],
-            "sys_argv_name": Path(sys.argv[0]).name,
-            "install": 'git' if is_git_dir() else 'pip' if is_pip_package() else 'other',
-            "os": ENVIRONMENT}
+            'sys_argv': sys.argv[0],
+            'sys_argv_name': Path(sys.argv[0]).name,
+            'install': 'git' if is_git_dir() else 'pip' if is_pip_package() else 'other',
+            'os': ENVIRONMENT}
         return event
 
     if SETTINGS['sync'] and \
             RANK in {-1, 0} and \
             Path(sys.argv[0]).name == 'yolo' and \
-            not is_pytest_running() and \
-            not is_github_actions_ci() and \
+            not TESTS_RUNNING and \
             ((is_pip_package() and not is_git_dir()) or
-             (get_git_origin_url() == "https://github.com/ultralytics/ultralytics.git" and get_git_branch() == "main")):
-
-        import hashlib
+             (get_git_origin_url() == 'https://github.com/ultralytics/ultralytics.git' and get_git_branch() == 'main')):
 
         import sentry_sdk  # noqa
-
         sentry_sdk.init(
-            dsn="https://f805855f03bb4363bc1e16cb7d87b654@o4504521589325824.ingest.sentry.io/4504521592406016",
+            dsn='https://f805855f03bb4363bc1e16cb7d87b654@o4504521589325824.ingest.sentry.io/4504521592406016',
             debug=False,
             traces_sample_rate=1.0,
             release=__version__,
             environment='production',  # 'dev' or 'production'
             before_send=before_send,
             ignore_errors=[KeyboardInterrupt, FileNotFoundError])
-        sentry_sdk.set_user({"id": SETTINGS['uuid']})
+        sentry_sdk.set_user({'id': SETTINGS['uuid']})
 
         # Disable all sentry logging
-        for logger in "sentry_sdk", "sentry_sdk.errors":
+        for logger in 'sentry_sdk', 'sentry_sdk.errors':
             logging.getLogger(logger).setLevel(logging.CRITICAL)
 
 
@@ -618,16 +591,17 @@ def set_settings(kwargs, file=USER_CONFIG_DIR / 'settings.yaml'):
 # Run below code on yolo/utils init ------------------------------------------------------------------------------------
 
 # Set logger
-set_logging(LOGGING_NAME)  # run before defining LOGGER
+set_logging(LOGGING_NAME, verbose=VERBOSE)  # run before defining LOGGER
 LOGGER = logging.getLogger(LOGGING_NAME)  # define globally (used in train.py, val.py, detect.py, etc.)
-if platform.system() == 'Windows':
+if WINDOWS:
     for fn in LOGGER.info, LOGGER.warning:
         setattr(LOGGER, fn.__name__, lambda x: fn(emojis(x)))  # emoji safe logging
 
 # Check first-install steps
-PREFIX = colorstr("Ultralytics: ")
+PREFIX = colorstr('Ultralytics: ')
 SETTINGS = get_settings()
 DATASETS_DIR = Path(SETTINGS['datasets_dir'])  # global datasets directory
 ENVIRONMENT = 'Colab' if is_colab() else 'Kaggle' if is_kaggle() else 'Jupyter' if is_jupyter() else \
     'Docker' if is_docker() else platform.system()
+TESTS_RUNNING = is_pytest_running() or is_github_actions_ci()
 set_sentry()
