@@ -91,6 +91,10 @@ def on_predict_batch_end(predictor):
     pass
 
 
+def on_predict_postprocess_end(predictor):
+    pass
+
+
 def on_predict_end(predictor):
     pass
 
@@ -130,6 +134,7 @@ default_callbacks = {
     # Run in predictor
     'on_predict_start': [on_predict_start],
     'on_predict_batch_start': [on_predict_batch_start],
+    'on_predict_postprocess_end': [on_predict_postprocess_end],
     'on_predict_batch_end': [on_predict_batch_end],
     'on_predict_end': [on_predict_end],
 
